SystemOrganization addCategory: #'Collections-Tests-BitmapCharacterSet'!

TestCase subclass: #BitmapCharacterSetTest
	instanceVariableNames: 'characters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Tests-BitmapCharacterSet'!

!BitmapCharacterSetTest methodsFor: 'enumerating' stamp: 'monty 9/1/2015 05:55'!
absentCharactersDo: aBlock
	| previousValue |

	previousValue := 0.
	self charactersDo: [:each |
		previousValue + 1 to: each asciiValue - 1 do: [:i |
			aBlock value: i asCharacter].
		previousValue := each asciiValue].

	previousValue + 1 to: self lastCodePoint - 1 do: [:i |
		aBlock value: i asCharacter].! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'JAAyer 1/20/2011 02:45'!
characterSetClass
	^ BitmapCharacterSet! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 9/1/2015 05:56'!
characters
	^ characters
		ifNil: [| writeStream previousValue offset |
			writeStream := (Array new: 250000) writeStream.
			previousValue := 0.
			writeStream nextPut: previousValue asCharacter.
			offset := 1.
			1 to: self lastCodePoint do: [:i |
				previousValue + offset = i
					ifTrue: [
						writeStream nextPut: i asCharacter.
						previousValue := i.
						offset :=
							offset = 8
								ifTrue: [1]
								ifFalse: [offset + 1]]].
			characters := writeStream contents]! !

!BitmapCharacterSetTest methodsFor: 'enumerating' stamp: 'monty 8/30/2015 06:56'!
charactersDo: aBlock
	self characters do: aBlock! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'JAAyer 1/20/2011 00:45'!
emptyCharacterSet
	^ self characterSetClass new! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/10/2016 10:22'!
lastCodePoint
	^ 16r10FFFF! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'JAAyer 1/20/2011 00:56'!
nonEmptyCharacterSet
	^ self characterSetClass newFrom: self characters! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 8/30/2015 05:13'!
testAdd
	| set |

	set := self emptyCharacterSet.
	self charactersDo: [:each |
		self
			deny: (set includes: each);
			assert: (set add: each) = each;
			assert: (set includes: each);
			assert: (set add: each) = each;
			assert: (set includes: each)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 14:18'!
testByteCharactersDo
	| set enumerated |

	set := self emptyCharacterSet.
	enumerated := OrderedCollection new.

	set byteCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set byteCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated notEmpty.
	enumerated withIndexDo: [:each :i |
		self
			assert: each asciiValue < 256;
			assert: each = (self characters at: i)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 11:25'!
testCapacity
	"Since the bitmap is allocated in bytes, the last byte can have excess capacity
	even when a specific capacity is specified."

	self
		assert: (self characterSetClass new: 0) capacity = 256;
		assert: (self characterSetClass new: 256) capacity = 256.
	257 to: 264 do: [:i |
		self assert: (self characterSetClass new: i) capacity = 264].
	265 to: 272 do: [:i |
		self assert: (self characterSetClass new: i) capacity = 272].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 4/5/2015 23:44'!
testCopy
	| set copy |

	set := self nonEmptyCharacterSet.
	copy := set copy.
	self
		assert: set = copy;
		deny: set == copy;
		assert: set byteCharacters = copy byteCharacters;
		deny: set byteCharacters == copy byteCharacters;
		assert: set wideCharacters = copy wideCharacters;
		deny: set wideCharacters == copy wideCharacters.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 14:18'!
testDo
	| set enumerated |

	set := self emptyCharacterSet.
	enumerated := OrderedCollection new.

	set do: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set do: [:each | enumerated addLast: each].
	self assert: enumerated size = self characters size.
	enumerated
		with: self characters
		do: [:enumeratedChar :expectedChar |
			self assert: enumeratedChar = expectedChar]! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'JAAyer 1/21/2011 17:51'!
testEquals
	self
		assert: self emptyCharacterSet = self emptyCharacterSet;
		assert: self nonEmptyCharacterSet = self nonEmptyCharacterSet;
		deny: self emptyCharacterSet = self nonEmptyCharacterSet;
		deny: self nonEmptyCharacterSet = self emptyCharacterSet! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 12:38'!
testHasWideCharacters
	| set |

	set := self emptyCharacterSet.
	self deny: set hasWideCharacters.

	set add: 255 asCharacter.
	self deny: set hasWideCharacters.

	set add: 256 asCharacter.
	self assert: set hasWideCharacters.

	set remove: 256 asCharacter.
	self deny: set hasWideCharacters.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 11:18'!
testHash
	self
		assert: self emptyCharacterSet hash = self emptyCharacterSet hash;
		assert: self nonEmptyCharacterSet hash = self nonEmptyCharacterSet hash! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 8/30/2015 04:59'!
testIncludes
	| set |

	set := self emptyCharacterSet.
	self
		charactersDo: [:each | self deny: (set includes: each)];
		absentCharactersDo: [:each | self deny: (set includes: each)].

	set := self nonEmptyCharacterSet.
	self
		charactersDo: [:each | self assert: (set includes: each)];
		absentCharactersDo: [:each | self deny: (set includes: each)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 11:45'!
testMaxCapacity
	| maxCapacity set |

	maxCapacity := self lastCodePoint + 1.

	set := self characterSetClass new: maxCapacity.
	self assert: set capacity = maxCapacity.

	set := self characterSetClass new: maxCapacity + 1.
	self assert: set capacity = maxCapacity.

	set := self characterSetClass new: maxCapacity * 2.
	self assert: set capacity = maxCapacity.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 10:25'!
testMaxCharacter
	| set |

	set := self emptyCharacterSet.
	self
		shouldnt: [set add: self lastCodePoint asCharacter]
		raise: Error.
	self assert: (set includes: self lastCodePoint asCharacter).
	self
		should: [set add: (self lastCodePoint + 1) asCharacter]
		raise: Error.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 8/30/2015 05:09'!
testNewFrom
	| set newSet |

	set := self characterSetClass newFrom: self characters.
	self assert: set class == self characterSetClass.
	self charactersDo: [:each | self assert: (set includes: each)].

	newSet := self characterSetClass newFrom: set.
	self
		assert: newSet = set;
		deny: newSet == set.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 12:46'!
testRemoveAll
	| set capacity byteCharactersSize wideCharactersSize|

	set := self nonEmptyCharacterSet.
	capacity := set capacity.
	byteCharactersSize := set byteCharacters size.
	wideCharactersSize := set wideCharacters size.
	self
		assert: set removeAll == set;
		assert: set size = 0;
		assert: set capacity = capacity;
		assert: set byteCharacters size = byteCharactersSize;
		assert: (set byteCharacters allSatisfy: [:each | each = false]);
		assert: set wideCharacters size = wideCharactersSize;
		assert: (set wideCharacters allSatisfy: [:each | each = 0]).! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 8/30/2015 05:12'!
testRemoveIfAbsent
	| set |

	set := self nonEmptyCharacterSet.
	self absentCharactersDo: [:each |
		self assert: (set remove: each ifAbsent: [nil]) isNil].
	self charactersDo: [:each |
		self assert: (set remove: each ifAbsent: [nil]) = each.
		self assert: (set remove: each ifAbsent: [nil]) isNil].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 9/1/2015 10:18'!
testSize
	| set size |

	set := self emptyCharacterSet.
	size := 0.
	self charactersDo: [:each |
		self assert: set size = size.
		set add: each.
		size := size + 1.
		self assert: set size = size.

		"Adding the same character shouldn't alter the size"
		set add: each.
		self assert: set size = size.].

	self charactersDo: [:each |
		set remove: each.
		size := size - 1.
		self assert: set size = size].	! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 10:33'!
testWideCharacters
	0 to: 7 do: [:i | | set character |
		set := self characterSetClass new.
		character := (256 + i) asCharacter.
		self assert: set wideCharacters isNil.
		set add: character.
		self assert: set wideCharacters first = (16r80 bitShift: i negated).
		set remove: character.
		self assert: set wideCharacters first = 0].
	0 to: 7 do: [:i | | set character |
		set := self characterSetClass new.
		character := (256 + 8 + i) asCharacter.
		self assert: set wideCharacters isNil.
		set add: character.
		self assert: set wideCharacters second = (16r80 bitShift: i negated).
		set remove: character.
		self assert: set wideCharacters second = 0]! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/10/2016 14:18'!
testWideCharactersDo
	| set enumerated wideCharOffset |

	set := self emptyCharacterSet.
	enumerated := OrderedCollection new.

	set wideCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set wideCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated notEmpty.
	wideCharOffset :=
		(self characters findFirst: [:each | each asciiValue >= 256]) - 1.
	enumerated withIndexDo: [:each :i |
		self
			assert: each asciiValue >= 256;
			assert: each = (self characters at: wideCharOffset + i)].! !
